@@ -1,110 +1,108 @@
 package Thread::Serialize;
 
-# Make sure we have version info for this module
-# Make sure we do everything by the book from now on
+$VERSION= '0.12';
 
-$VERSION = '0.09';
+# be as strict as possble
 use strict;
 
-# Make sure we only load things that we need when we need it
-
+# modules that we need
 use load;
-
-# Make sure we can freeze and thaw
-
 use Storable ();
 
-# Execute external perl to obtain Storable signature (saves memory here)
+# use ourselves to determine signature
+our $iced;
+if ( $Thread::Serialize::no_external_perl ) {
+    $iced= unpack( 'l',Storable::freeze( [] ));
+    $Thread::Serialize::no_external_perl= 'Signature obtained locally';
+}
 
-open( my $handle,
- qq($^X -MStorable -e "print unpack('l',Storable::freeze( [] ))" | )
-) or die "Cannot determine Storable signature\n";
-our $iced = <$handle>;
-undef( $handle );
-
-# Satisfy -require-
+# use external perl to create signature
+else {
+    open( my $handle,
+      qq($^X -MStorable -e "print unpack('l',Storable::freeze( [] ))" | )
+    ) or die "Cannot determine Storable signature\n";
+    $iced= readline $handle;
+}
 
+# satisfy -require-
 1;
 
 # The following subroutines are loaded on demand only
 
 __END__
 
-#---------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+# freeze
+#
+# Freeze given parameters into a single scalar value
+#
 #  IN: 1..N parameters to freeze
 # OUT: 1 frozen scalar
 
 sub freeze {
 
-# If we have at least one element in the list
-#  For all of the elements
-#   Return truly frozen version if something special
-#  Return the values contatenated with null bytes
-# Else (empty list)
-#  Return undef value
-
+    # something to be frozen
     if (@_) {
+
+        # any of the parameters is special, so really freeze
         foreach (@_) {
             return Storable::freeze( \@_ ) if !defined() or ref() or m#\0#;
         }
-        return join( "\0",@_ );
-    } else {
-        return;
+
+        # just concatenate with null bytes (WAY faster)
+        return join( "\0", @_ );
     }
+
+    # no parameters, so just undef
+    return;
 } #freeze
 
-#---------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 #  IN: 1 frozen scalar to defrost
 # OUT: 1..N thawed data structure
 
 sub thaw {
 
-# Return now if nothing to return or not interested in result
-
-    return unless defined( $_[0] ) and defined( wantarray );
-
-# If we're interested in a list
-#  Return thawed list from frozen info if frozen
-#  Return list split from a normal string
-# Elseif we have frozen stuff (and we want a scalar)
-#  Thaw the list and return the first element
-# Else (not frozen and we want a scalar)
-#  Look for the first nullbyte and return string until then if found
-#  Return the string
+    # nothing here or not interested
+    return if !defined( $_[0] ) or !defined(wantarray);
 
+    # return as list
     if (wantarray) {
-        return @{Storable::thaw( $_[0] )} if (unpack( 'l',$_[0] )||0) == $iced;
-        split( "\0",$_[0] )
-    } elsif ((unpack( 'l',$_[0] )||0) == $iced) {
-        Storable::thaw( $_[0] )->[0];
-    } else {
-	return $1 if $_[0] =~ m#^([^\0]*)#;
-        $_[0];
+        return ( unpack( 'l', $_[0] ) || 0 ) == $iced
+          ? @{ Storable::thaw( $_[0] ) }
+          : split "\0", $_[0];
     }
+
+    # scalar, really frozen, return first value
+    elsif ( ( unpack( 'l',$_[0] ) || 0 ) == $iced ) {
+        return Storable::thaw( $_[0] )->[0];
+    }
+
+    # return first part of scalar
+    return $_[0] =~ m#^([^\0]*)# ? $1 : $_[0];
 } #thaw
 
-#---------------------------------------------------------------------------
-
-# standard Perl features
-
-#---------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+#
+# Standard Perl Features
+#
+#-------------------------------------------------------------------------------
 
 sub import {
-
-# Lose the class
-# Obtain the namespace
-# Obtain the names of the subroutines to export
-# Allow for dirty tricks
-# Export all subroutines specified
-
     shift;
-    my $namespace = caller().'::';
-    @_ = qw(freeze thaw) unless @_;
+
+    # determine namespace and subs
+    my $namespace= caller().'::';
+    @_= qw( freeze thaw ) if !@_;
+
+    # do our exports
     no strict 'refs';
-    *{$namespace.$_} = \&$_ foreach @_;
+    *{$namespace.$_}= \&$_ foreach @_;
+
+    return;
 } #import
 
-#---------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 
 __END__
 
@@ -121,6 +119,10 @@ Thread::Serialize - serialize data-structures between threads
   my $frozen = freeze( any data structure );
   any data structure = thaw( $frozen );
 
+=head1 VERSION
+
+This documentation describes version 0.12.
+
 =head1 DESCRIPTION
 
                   *** A note of CAUTION ***
@@ -181,6 +183,24 @@ however revealed that the overhead of the compiling single routines is not
 much more (and sometimes a lot less) than the overhead of cloning a Perl
 interpreter with a lot of subroutines pre-loaded.
 
+To reduce the number of modules and subroutines loaded, an external Perl
+interpreter is started to determine the Storable signature at compile time.
+In some situations this may cause a problem: please set the
+C<$Thread::Serialize::no_external_perl> variable to a true value at compile
+time B<before> loading Thread::Serialize if this causes a problem.
+
+ BEGIN { $Thread::Serialize::no_external_perl= 1 }
+ use Thread::Serialize;
+
+=head1 KNOWN ISSUES
+
+=head2 Embedded Perls
+
+Philip Monsen reported that in the case of an embedded Perl interpreter (e.g.
+in a C program), the use of an external executor to determine the Storable
+signature, causes problems.  This has been fixed by introducing the global
+variable C<$Thread::Serialize::no_external_perl> (see L<OPTIMIZATIONS>).
+
 =head1 AUTHOR
 
 Elizabeth Mattijsen, <liz@dijkmat.nl>.
@@ -189,9 +209,9 @@ Please report bugs to <perlbugs@dijkmat.nl>.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2002-2003 Elizabeth Mattijsen <liz@dijkmat.nl>. All rights
-reserved.  This program is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself.
+Copyright (c) 2002, 2003, 2004, 2010, 2012 Elizabeth Mattijsen <liz@dijkmat.nl>.
+All rights reserved.  This program is free software; you can redistribute it
+and/or modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
